#extension GL_OES_EGL_image_external : require

precision mediump float;


const float kIntensityFactor = 1.0;
const float kXrayDimension = 17.0;
const float kSubDimension = 16.0;
const float kXraySize = kXrayDimension * kXrayDimension;

vec2 colorCubeTextureCoordinate(in vec3 rgbIndex)
{
    return vec2(rgbIndex.r / kSubDimension,
                (rgbIndex.g + rgbIndex.b * kXrayDimension) / (kXraySize - 1.0));
}

vec4 lerp(in vec4 value1, in vec4 value2, in float diff)
{
    return(mix(value1, value2, diff));
}

vec4 colorLookup(in vec3 rgb, in sampler2D colorCube)
{
    vec3 floorRGBindex = vec3(floor(rgb.r * kSubDimension),
                              floor(rgb.y * kSubDimension),
                              floor(rgb.b * kSubDimension));

    vec4 p000 = texture2D(colorCube,
                          colorCubeTextureCoordinate(floorRGBindex));
    vec4 p001 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r,
                                                          floorRGBindex.g,
                                                          floorRGBindex.b + 1.0)));
    vec4 p010 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r,
                                                          floorRGBindex.g + 1.0,
                                                          floorRGBindex.b)));
    vec4 p011 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r,
                                                          floorRGBindex.g + 1.0,
                                                          floorRGBindex.b + 1.0)));
    vec4 p100 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0,
                                                          floorRGBindex.g,
                                                          floorRGBindex.b)));
    vec4 p101 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0,
                                                          floorRGBindex.g,
                                                          floorRGBindex.b + 1.0)));
    vec4 p110 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0,
                                                          floorRGBindex.g + 1.0,
                                                          floorRGBindex.b)));
    vec4 p111 = texture2D(colorCube,
                          colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0,
                                                          floorRGBindex.g + 1.0,
                                                          floorRGBindex.b + 1.0)));

    float x0 = floorRGBindex.x / kSubDimension;
    float y0 = floorRGBindex.y / kSubDimension;
    float z0 = floorRGBindex.z / kSubDimension;

    float xDiff = (rgb.r - x0) * kSubDimension;
    float yDiff = (rgb.g - y0) * kSubDimension;
    float zDiff = (rgb.b - z0) * kSubDimension;

    vec4 c00 = lerp(p000, p100, xDiff);
    vec4 c10 = lerp(p010, p110, xDiff);
    vec4 c01 = lerp(p001, p101, xDiff);
    vec4 c11 = lerp(p011, p111, xDiff);

    vec4 c0 = lerp(c00, c10, yDiff);
    vec4 c1 = lerp(c01, c11, yDiff);

    vec4 c = lerp(c0, c1, zDiff);
    return c;
}

varying mediump vec2 vTextureCoord;

uniform samplerExternalOES sImageTexture;
uniform sampler2D sColorCubeTexture;

void main()
{
    vec4 pixel = texture2D(sImageTexture, vTextureCoord);
    gl_FragColor = colorLookup(pixel.rgb, sColorCubeTexture);
}
