#version 300 es

precision mediump float;

// The assumption is the laplacian at iLevel + 1 and above are already calculated.
// This method reconstructs the gaussian at iLevel + 1 using these laplacians and
// output the laplacian at iLevel using the difference of sImageTexture at iLevel
// and the reconstructed gaussian at iLevel + 1.
// The reason for doing it this way is the sLaplacianTexture texture is stored
// at a much higher precision than the sImageTexture.

uniform sampler2D sImageTexture;     // inputImage texture
uniform sampler2D sLaplacianTexture; // partially constructed laplacian.
uniform int iLevel;
uniform int iLevelCount;
in vec2 vTextureCoord;
out vec4 FragColor;

void main() {
    vec3 recRGB = textureLod(sImageTexture, vTextureCoord, float(iLevelCount - 1)).rgb;
    for (int l = iLevel + 1; l < iLevelCount - 1; l++) {
        recRGB += textureLod(sLaplacianTexture, vTextureCoord, float(l)).rgb;
    }
    FragColor = vec4(textureLod(sImageTexture, vTextureCoord, float(iLevel)).rgb - recRGB, 1.0);
}

