#version 300 es

precision mediump float;

uniform sampler2D sMaskTexture;  // mask

in vec2 vTextureCoord;
out vec4 FragColor;

void main() {
    const float M_PI = 3.14159;
    const int numSteps = 100;
    const int numAngles = 16;
    const float MASK_LOD = 5.0;   // access the mask at this level.
  
    float stepSize = 1.0 / float(numSteps);  // radius increment.
    float angleSize = 1.0 / float(numAngles);

    // search for a patch that isn't masked out in concentric rings
    // of increasing distance
    for (int s = 1; s < numSteps; s++) {
        for (int a = 0; a < numAngles; a++) {
            // FIXME: we might want to randomize the angle here
            float angle = M_PI * float(a) * angleSize;
            vec2 offset = vec2(cos(angle), sin(angle)) * (float(s) * stepSize);
            if (textureLod(sMaskTexture, vTextureCoord + offset, MASK_LOD).r == 0.0) {
                // we found a patch position that isn't masked out!
                FragColor = vec4(offset.x, offset.y, 0.0, 1.0);
                return;
            }
        }
    }
    // no patch found. so we set FragColor to 0.
    FragColor = vec4(0.0, 0.0, 0.0, 1.0);
}