#version 300 es

precision mediump float;

uniform sampler2D sImageTexture; // inputImage texture
uniform sampler2D uTexture1; // llpImage texture
uniform sampler2D uTexture2; // inputImageLuminance texture

in vec2 vTextureCoord;
uniform float uFloat0; //intensity
const float cMinimumLuminanceValue = 0.000449; // 1.0 / 256.0;
const vec3 cLuminanceVector = vec3(0.2989, 0.5866, 0.1145);
out vec4 FragColor;

void main() {
  vec4 inputImage = texture(sImageTexture, vTextureCoord);
  float inputLum = dot(inputImage.rgb, cLuminanceVector);
  if (inputLum < cMinimumLuminanceValue) {
    FragColor = inputImage;
  }

  vec4 inputImageLuminance = texture(uTexture2, vTextureCoord);
  if (inputImageLuminance.r < cMinimumLuminanceValue) {
    FragColor = inputImage;
  }

  float factor = inputLum / inputImageLuminance.r;
  vec4 llpImage = texture(uTexture1, vTextureCoord);
  float outputLum = inputLum + uFloat0 * factor * (llpImage.r - inputImageLuminance.r);

  float red = outputLum * inputImage.r / inputLum;
  float green = outputLum * inputImage.g / inputLum;
  float blue = outputLum * inputImage.b / inputLum;

  FragColor = vec4(red, green, blue, inputImage.a);
}
