#version 300 es
precision highp float;

const float kIntensityFactor = 1.0;
const float kXrayDimension = 17.0;
const float kSubDimension = 16.0;
const float kXraySize = kXrayDimension * kXrayDimension;

const float cMinimumLuminanceValue = 0.000449; // 1.0 / 256.0;
const vec3 cLuminanceVector = vec3(0.2989, 0.5866, 0.1145);

in vec2 vTextureCoord;
uniform float uFloat0; //intensity
uniform sampler2D sColorCubeTexture;
uniform sampler2D sImageTexture;
uniform sampler2D uTexture1; // llpImage texture
uniform sampler2D uTexture2; // inputImageLuminance texture
out vec4 FragColor;

vec2 colorCubeTextureCoordinate(in vec3 rgbIndex)
{
    return vec2(rgbIndex.r / kSubDimension, (rgbIndex.g + rgbIndex.b * kXrayDimension) / (kXraySize - 1.0));
}

vec4 lerp(in vec4 value1, in vec4 value2, in float diff)
{
    return(mix(value1, value2, diff));
}

vec4 colorLookup(in vec4 rgb, in sampler2D colorCube)
{
    rgb = clamp(rgb, 0.0, 1.0);
    vec3 floorRGBindex = vec3(floor(rgb.r * kSubDimension), floor(rgb.g * kSubDimension), floor(rgb.b * kSubDimension));

    vec4 p000 = texture(colorCube, colorCubeTextureCoordinate(floorRGBindex));
    vec4 p001 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g, floorRGBindex.b + 1.0)));
    vec4 p010 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g + 1.0, floorRGBindex.b)));
    vec4 p011 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g + 1.0, floorRGBindex.b + 1.0)));
    vec4 p100 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g, floorRGBindex.b)));
    vec4 p101 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g, floorRGBindex.b + 1.0)));
    vec4 p110 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g + 1.0, floorRGBindex.b)));
    vec4 p111 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g + 1.0, floorRGBindex.b + 1.0)));
    float xDiff = (rgb.r * kSubDimension) - floorRGBindex.x;
    float yDiff = (rgb.g * kSubDimension) - floorRGBindex.y;
    float zDiff = (rgb.b * kSubDimension) - floorRGBindex.z;
    vec4 c00 = lerp(p000, p100, xDiff);
    vec4 c10 = lerp(p010, p110, xDiff);
    vec4 c01 = lerp(p001, p101, xDiff);
    vec4 c11 = lerp(p011, p111, xDiff);
    vec4 c0 = lerp(c00, c10, yDiff);
    vec4 c1 = lerp(c01, c11, yDiff);
    vec4 c = lerp(c0, c1, zDiff);
    return c;
}



void main()
{
 vec4 inputImage = texture(sImageTexture, vTextureCoord);
  float inputLum = dot(inputImage.rgb, cLuminanceVector);
  if (inputLum < cMinimumLuminanceValue) {
    FragColor = inputImage;
  }

  vec4 inputImageLuminance = texture(uTexture2, vTextureCoord);
  if (inputImageLuminance.r < cMinimumLuminanceValue) {
    FragColor = inputImage;
  }

  float factor = inputLum / inputImageLuminance.r;
  vec4 llpImage = texture(uTexture1, vTextureCoord);
  float outputLum = inputLum + uFloat0 * factor * (llpImage.r - inputImageLuminance.r);

  float red = outputLum * inputImage.r / inputLum;
  float green = outputLum * inputImage.g / inputLum;
  float blue = outputLum * inputImage.b / inputLum;

  vec4 inputRGB = vec4(red, green, blue, inputImage.a);
  FragColor = colorLookup(inputRGB, sColorCubeTexture);
}
