#version 300 es

precision highp float;

uniform sampler2D sImageTexture; // inputImage texture

in vec2 vTextureCoord;
uniform float uStrength;
uniform float uTime;

out vec4 FragColor;

    vec4 desaturate(vec4 color, float strength) {
        
        mat3 yuv2rgb = mat3(1,        1,        1,
                                    0,       -0.187324, 1.8556,
                                    1.5748 , -0.468124, 0);
        
        mat3 yuv2rgbInverse = mat3(0.21259992476445786239, -0.1145720655122105313, 0.50000004777466480671,
                                           0.71520016388621015299, -0.3854279822624542753, -0.45415301237376819468,
                                           0.072199911349331984641, 0.50000004777466480672, -0.04584703540089661204);
        
        vec3 yuv = yuv2rgbInverse * color.rgb;
        yuv.yz *= 1.0 - strength;
        return vec4(yuv2rgb * yuv, color.a);
    }
    
    
    float sat(float t) {
        return clamp(t, 0.0, 1.0);
    }
    
    vec2 satVec(vec2 t) {
        return clamp(t, 0.0, 1.0);
    }
    
    
    // remaps inteval [a;b] to [0;1]
    float remap(float t, float a, float b) {
        return sat((t - a) / (b - a));
    }
    
    // note: t=[0;0.5;1], y=[0;1;0]
    float linterp(float t) {
        return sat(1.0 - abs(2.0*t - 1.0));
    }
    
    vec3 spectrum_offset(float t) {
        float t0 = 3.0 * t - 1.5;
        return clamp(vec3(-t0, 1.0-abs(t0), t0), 0.0, 1.0);
    }
    
    // note: [0;1]
    float rand(vec2 n) {
        return fract(sin(dot(n.xy, vec2(12.9898, 78.233)))* 43758.5453);
    }
    
    // note: [-1;1]
    float srand(vec2 n) {
        return rand(n) * 2.0 - 1.0;
    }
    
    float mytrunc(float x, float num_levels) {
        return floor(x*num_levels) / num_levels;
    }
    
    vec2 mytruncVec(vec2 x, float num_levels) {
       return floor(x*num_levels) / num_levels;
    }

    void main() {
        vec2 uv = vTextureCoord;

        float time = mod(uTime + 0.1234, 32.0);
        float glitchStrength = uStrength;
        
        float gnm = sat(glitchStrength);
        float rnd0 = rand(mytruncVec(vec2(time, time), 6.0));
        float r0 = sat((1.0-gnm)*0.7 + rnd0);
        float rnd1 = rand(vec2(mytrunc( uv.x, 10.0*r0), time)); //horz
        float r1 = 0.5 - 0.5 * gnm + rnd1;
        r1 = 1.0 - max(0.0, ((r1<1.0) ? r1 : 0.9999999)); //note: weird ass bug on old drivers
        float rnd2 = rand(vec2(mytrunc(uv.y, 40.0*r1), time)); //vert
        float r2 = sat(rnd2);
        
        float rnd3 = rand(vec2(mytrunc(uv.y, 10.0*r0), time));
        float r3 = (1.0-sat(rnd3+0.8)) - 0.1;
        
        float pxrnd = rand(uv + time);
        
        float ofs = 0.05 * r2 * glitchStrength * (rnd0 > 0.5 ? 1.0 : -1.0);
        ofs += 0.5 * pxrnd * ofs;
        
        uv.y += 0.1 * r3 * glitchStrength;
        
        const int sampleCount = 5;
        const float sampleCountInverse = 1.0 / float(sampleCount);
        
        vec4 sum = vec4(0.0);
        vec3 wsum = vec3(0.0);
        for(int i=0; i < sampleCount; i++) {
            float t = float(i) * sampleCountInverse;
            uv.x = sat(uv.x + ofs * t);
            vec4 samplecol = desaturate(texture(sImageTexture, uv, -10.0), glitchStrength);
            vec3 s = spectrum_offset(t);
            samplecol.rgb = samplecol.rgb * s;
            sum += samplecol;
            wsum += s;
        }
        sum.rgb /= wsum;
        sum.a *= sampleCountInverse;
        
        FragColor = vec4(sum.rgb, sum.a);
    }
