#version 300 es

precision mediump float;

uniform sampler2D sImageTexture; // inputImage texture

uniform sampler2D sMaskTexture;  // mask

in vec2 vTextureCoord;
uniform int iLevel;
out vec4 FragColor;

void main() {
    // pixel size in the normalized cordinate for input image.
    vec2 pixel = 1.0 / vec2(textureSize(sImageTexture, iLevel));
    float l = float(iLevel);

    // RGB values of the sImageTexture mipmap pyramid at the level.
    vec3 rgb = textureLod(sImageTexture, vTextureCoord, l).rgb;

    // only red channel on the mask is used.
    float mask = textureLod(sMaskTexture, vTextureCoord, l).r;

    // If the current pixel is masked.
    if (mask > 0.0) {
        const int windowSize = 3;
        int dist = windowSize * windowSize * 4;
        int count = 0;

        for (int y = -windowSize; y <= windowSize; y++) {
            for (int x = -windowSize; x <= windowSize; x++) {
                vec2 offset = vec2(x, y) * pixel;
                float m = textureLod(sMaskTexture, vTextureCoord + offset, l).r;
                if (m == 0.0) {
                    int d = x*x + y*y;
                    if (d < dist) {
                        rgb = vec3(0.0);
                        dist = d;
                        count = 0;
                    }
                    if (d == dist) {
                        rgb += textureLod(sImageTexture, vTextureCoord + offset, l).rgb;
                        count++;
                    }
                }
            }
        }
        if (count > 1) {
            rgb /= float(count);
        }
    }
    FragColor = vec4(rgb, 1.0);
}
