#version 300 es

precision highp float;

uniform sampler2D sImageTexture; // inputImage texture

in vec2 vTextureCoord;
uniform float uAspectX;
uniform int uMirrorCount;
uniform float uTime;

out vec4 FragColor;

float mirrorCoord(float a) {
    if (a >= 0.0 && a <= 1.0) return a;
    return 1.0 - (a - (floor(a)));
}
vec2 mirrorVec(vec2 position) {
    return vec2(mirrorCoord(position.x), mirrorCoord(position.y));
}

vec4 getSampleRepeatMirror(sampler2D image, vec2 position) {
    return texture(image, mirrorVec(position));
}

mat2 Rotation2D(float angle) {
    float c = cos(angle);
    float s = sin(angle);
    return mat2(c,-s,
                s, c);
}

float rand(vec2 v) {
    return fract(sin(dot(v, vec2(12.9898,78.233))) * 43758.5453);
}

vec3 blendOverlay(vec3 base, vec3 blend, float opacity) {
    return mix(1.0 - 2.0 * (1.0 - base) * (1.0 - blend), 2.0 * base * blend, step(base, vec3(0.5))) * opacity + base * (1.0 - opacity);
}

void main() {
    const float M_PI = 3.141592;
	const float CHROMA_AMOUNT = .075;
	const float GRAIN_AMOUNT = .1;

    vec2 aspect = vec2(uAspectX, 1);
    vec2 uv = (vTextureCoord - 0.5) * aspect;
    float sliceAngle = M_PI / float(uMirrorCount);
    for (int i=0; i < uMirrorCount; i++) {
      float angle = float(i) * sliceAngle;
      mat2 R = Rotation2D(angle);
      uv = R * uv;
      uv.x = abs(uv.x);
      uv = inverse(R) * uv;
    }

    float centerDistance = distance(vTextureCoord, vec2(.5, .5));
    float distanceCubed = centerDistance * centerDistance * centerDistance;

    vec2 chromaVec = vec2(CHROMA_AMOUNT, 0.0) * distanceCubed;
    vec2 adjustedUV = uv / aspect + 0.5;

    vec4 fragmentColor = getSampleRepeatMirror(sImageTexture, adjustedUV);
    fragmentColor.r = getSampleRepeatMirror(sImageTexture, adjustedUV + chromaVec).r;
    fragmentColor.g = getSampleRepeatMirror(sImageTexture, adjustedUV - chromaVec).g;

    float normalizedRand = rand(mirrorVec(vTextureCoord + uTime));
    FragColor = vec4(blendOverlay(fragmentColor.rgb, vec3(normalizedRand), GRAIN_AMOUNT), 1);
}