#version 300 es

precision mediump float;

uniform sampler2D sImageTexture;          // inputImage texture
uniform sampler2D sOverlayImageTexture;   // overlay 1
uniform sampler2D sOverlayImageTexture2;  // overlay 2

in vec2 vTextureCoord;
in vec2 vOverlayTextureCoord;
in vec2 vOverlayTextureCoord2;

uniform int uBlendMode;                 // blend mode
uniform float uStrength;                // intensity for overlay 1
uniform float uStrength2;               // intensity for overlay 2

out vec4 FragColor;

vec3 blendHardLight(vec3 base, vec3 overlay, float strength) {
    vec3 blend =  mix(
        2.0f * base * overlay,
        1.0f - 2.0f * (1.0f - base) * (1.0f - overlay),
        step(0.5f, overlay)
    );
    return mix(base, blend, strength);
}

vec3 blendMultiply(vec3 base, vec3 overlay, float strength) {
    vec3 blend = base * overlay;
    return mix(base, blend, strength);
}

vec3 blendScreen(vec3 base, vec3 overlay, float strength) {
    vec3 blend =  1.0f - (1.0f - base) * (1.0f - overlay);
    return mix(base, blend, strength);
}

void main() {
    vec4 originalColor = texture(sImageTexture, vTextureCoord);
    vec4 overlayColor = texture(sOverlayImageTexture, vOverlayTextureCoord);
    vec4 overlay2Color = texture(sOverlayImageTexture2, vOverlayTextureCoord2);
    float strength = clamp(uStrength, 0.0f, 1.0f);
    vec4 color = originalColor;

    if (uBlendMode == 0) {
        if (strength != 0.0) {
            color = vec4(blendHardLight(originalColor.xyz, overlayColor.xyz, strength), originalColor.w);
        }
        if (uStrength2 != 0.0) {
            color = vec4(blendHardLight(color.xyz, overlay2Color.xyz, uStrength2), color.w);
        }
    } else if (uBlendMode == 1) {
        if (strength != 0.0) {
            color = vec4(blendMultiply(originalColor.xyz, overlayColor.xyz, strength), originalColor.w);
        }
        if (uStrength2 != 0.0) {
            color = vec4(blendMultiply(color.xyz, overlay2Color.xyz, uStrength2), color.w);
        }
    } else if (uBlendMode == 2) {
        if (strength != 0.0) {
            color = vec4(blendScreen(originalColor.xyz, overlayColor.xyz, strength), originalColor.w);
        }

        if (uStrength2 != 0.0) {
            color = vec4(blendScreen(color.xyz, overlay2Color.xyz, uStrength2), color.w);
        }
    }

    FragColor = color;
}