#version 300 es

precision mediump float;

uniform sampler2D sImageTexture;
uniform sampler2D sMaskTexture;
uniform sampler2D sColorCubeTexture;

in vec2 vTextureCoord;
out vec4 FragColor;

const float kIntensityFactor = 1.0;
const float kXrayDimension = 17.0;
const float kSubDimension = 16.0;
const float kXraySize = kXrayDimension * kXrayDimension;

vec2 colorCubeTextureCoordinate(in vec3 rgbIndex)
{
    return vec2(rgbIndex.r / kSubDimension, (rgbIndex.g + rgbIndex.b * kXrayDimension) / (kXraySize - 1.0));
}

vec4 lerp(in vec4 value1, in vec4 value2, in float diff)
{
    return(mix(value1, value2, diff));
}

vec4 colorLookup(in vec4 rgb, in sampler2D colorCube)
{
    rgb = clamp(rgb, 0.0, 1.0);
    vec3 floorRGBindex = vec3(floor(rgb.r * kSubDimension), floor(rgb.g * kSubDimension), floor(rgb.b * kSubDimension));
    vec4 p000 = texture(colorCube, colorCubeTextureCoordinate(floorRGBindex));
    vec4 p001 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g, floorRGBindex.b + 1.0)));
    vec4 p010 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g + 1.0, floorRGBindex.b)));
    vec4 p011 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r, floorRGBindex.g + 1.0, floorRGBindex.b + 1.0)));
    vec4 p100 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g, floorRGBindex.b)));
    vec4 p101 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g, floorRGBindex.b + 1.0)));
    vec4 p110 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g + 1.0, floorRGBindex.b)));
    vec4 p111 = texture(colorCube, colorCubeTextureCoordinate(vec3(floorRGBindex.r + 1.0, floorRGBindex.g + 1.0, floorRGBindex.b + 1.0)));
    float xDiff = (rgb.r * kSubDimension) - floorRGBindex.x;
    float yDiff = (rgb.g * kSubDimension) - floorRGBindex.y;
    float zDiff = (rgb.b * kSubDimension) - floorRGBindex.z;
    vec4 c00 = lerp(p000, p100, xDiff);
    vec4 c10 = lerp(p010, p110, xDiff);
    vec4 c01 = lerp(p001, p101, xDiff);
    vec4 c11 = lerp(p011, p111, xDiff);
    vec4 c0 = lerp(c00, c10, yDiff);
    vec4 c1 = lerp(c01, c11, yDiff);
    vec4 c = lerp(c0, c1, zDiff);
    return c;
}

void main() {
    vec4 inputPixel = texture(sImageTexture, vTextureCoord);
    vec4 modified = colorLookup(inputPixel, sColorCubeTexture);
    float mask = texture(sMaskTexture, vTextureCoord).a;
    FragColor = clamp(modified * mask +  inputPixel *(1.0 - mask), 0.0, 1.0);
}
