#version 300 es

precision mediump float;

uniform sampler2D sImageTexture; // inputImage texture
uniform sampler2D sOverlayImageTexture; // overlayImage texture

in vec2 vTextureCoord;
uniform float uStrength; //intensity
out vec4 FragColor;

vec3 blendOverlay(vec3 base, vec3 blend, float opacity) {
    return mix(
       1.0 - 2.0 * (1.0 - base) * (1.0 - blend), 2.0 * base * blend, step(base, vec3(0.5))
    ) * opacity + base * (1.0 - opacity);
}

vec3 blendLinearLight(vec3 base, vec3 blend, float opacity) {
    return (base + (2.0 * blend) - 1.0) * opacity + base * (1.0 - opacity);
}


void main() {

    vec4 originalColor = texture(sImageTexture, vTextureCoord);
    if (uStrength == 0.0) {
        FragColor = originalColor;
    } else {
        vec4 overlayColor = texture(sOverlayImageTexture, vTextureCoord);
        FragColor = vec4(blendOverlay(originalColor.xyz, overlayColor.xyz, uStrength), originalColor.w);
    }
}
